//  *****************************************************************************************************************
//  *                                                                                                               *
//  *                                         SpikenzieLabs.com                                                     *
//  *                                                                                                               *
//  *                                        Drum Kit - Kit OCHO                                                    *
//  *                                                                                                               *
//  *                                                                                                               *
//  *****************************************************************************************************************
//
//  BY: MARK DEMERS Copywrite 2014
//  Jan. 2014
//  VERSION: 3.0
//
//  DESCRIPTION:
//  Analog input used to sense piezo drum hits then send notes serially, or via MIDI port.
//  NEW - DKK-OCHO with upto 8 drum pads and with Field Programmer built in.
//  
//  Required - Hardware:
//  1. Drum kit - kit Ocho (From SpikenzieLabs.com)
//  2. MIDI output device (either computer or MIDI sampler)
//  3. Cables - Either MIDI-MIDI, MIDI-USB or USB-TTL serial (eg. FTDI Buddy http://goo.gl/gCHXqr or any FTDI style cable)
//
//  Required - Software:
//  1. Serial MIDI converter (or optional hardware MIDI input devide)
//  2. Garage Band, Ableton Live etc ... (or optional hardware MIDI input devide)
//  3. Arduino IDE (Used for tuning the Ocho pad sensitivity)
//
// LEGAL:
// This code is provided as is. No guaranties or warranties are given in any form. It is your responsibilty to 
// determine this codes suitability for your application.
//
// Changes:
// 

//*******************************************************************************************************************
// User settable variables			
//*******************************************************************************************************************

unsigned char PadNote[8] = {
  60,61,62,63,64,65,66,67};                              // MIDI notes from 0 to 127 (Mid C = 60)

int PadCutOff[8] = {
  500,500,500,500,500,500,500,500};                      // Minimum Analog value to cause a drum hit

int MaxPlayTime[8] = {
  40,40,40,40,40,40,40,40};                              // Cycles before a 2nd hit is allowed

#define  midichannel	0;                               // MIDI channel from 0 to 15 (+1 in "real world")

boolean VelocityFlag  = false;                           // Velocity ON (true) or OFF (false)

// Number of Drum Pads used
#define NumPads 8                                        // Number of drum pads used (Up to 8)
// NOTE: Drum pads are numbered from left to right when facing the DKK Ocho's screw terminals.
// If you are using less than 8 pads, connected the pads starting from left to right.

//*******************************************************************************************************************
// Internal Use Variables and Constants
//*******************************************************************************************************************

boolean activePad[8] = {
  0,0,0,0,0,0,0,0};                   // Array of flags of pad currently playing
int PinPlayTime[8] = {
  0,0,0,0,0,0,0,0};                     // Counter since pad started to play

unsigned char status;

int pin = 0;     
int hitavg = 0;
int val = 0;

int RPGpin =0 ;

#define LEDspin  8                                    // DKK Ocho's "Roadie" Led Pins
#define BUTTONpin  9                                  // DKK Ocho's Roadie Button Pins
#define COL1pin  5
#define COL2pin  6
#define COL3pin  7

int ColSelect = 1;                                  // Select pin of LED / Button interface

boolean ModePress = false;
boolean ModePressWAIT = false;
boolean UpPress = false;
boolean DownPress = false;

boolean PlayLED = true;
boolean NoteLED = false;
boolean SelectLED = false;

int SampTime = 400;                                  // Program cycles to play sample sounds
int SampCounter = 0;
int SampPlaying = 0;
boolean SampON = false;
int mode = 1;

//*******************************************************************************************************************
// Setup			
//*******************************************************************************************************************

#include <EEPROM.h>

void setup() 
{

  digitalWrite(LEDspin, HIGH);                     // Set internal pull-up to high on button pins
  digitalWrite(COL1pin, LOW);
  digitalWrite(COL2pin, LOW);
  digitalWrite(COL3pin, LOW);

  pinMode(LEDspin, OUTPUT);                         // Set Button Pins to inputs
  pinMode(BUTTONpin, INPUT);
  pinMode(COL1pin, OUTPUT);
  pinMode(COL2pin, OUTPUT);   
  pinMode(COL3pin, OUTPUT);

  delay(250);

  Serial.begin(31250);                                  // connect the serial port Use 31250 for MIDI [default serial DKK 57600]


  val = EEPROM.read(9);                                 // Position 9 in EEPROM used to see if it has Note data 
  if(val != 78)                                         // (if it is not "78" then we never filled EEPROM, do it now.)
  {
    FILLEEPROM();
  } 
  else
  {
    GETNOTESFROMEEPROM();
  }

  digitalWrite(LEDspin, LOW);                      // Turn on LED for column 1 (Play LED)
  digitalWrite(COL1pin, HIGH);

}

//*******************************************************************************************************************
// Main Program			
//*******************************************************************************************************************

void loop() 
{

  if(mode != 1)
  {
    switch (ColSelect)                                         // Check Buttons / Update LED feedback
    {
    case 1:  
      digitalWrite(LEDspin, HIGH);
      digitalWrite(COL2pin, LOW);
      digitalWrite(COL3pin, LOW);

      digitalWrite(COL1pin, HIGH);

      val = digitalRead(BUTTONpin);
      if(val == 1)
      {
        do
        {
          val = digitalRead(BUTTONpin);
          if(val == 0)
          {
            ModePress = true;
          }
        }
        while(val == 1);

        PlayLED = false;
        NoteLED = true;
      }

      ColSelect = 2;

      break;

    case 2:  
      digitalWrite(LEDspin, HIGH);
      digitalWrite(COL1pin, LOW);
      digitalWrite(COL3pin, LOW);

      digitalWrite(COL2pin, HIGH);

      val = digitalRead(BUTTONpin);
      if(val == 1)
      {
        UpPress = true;
        do
        {
          val = digitalRead(BUTTONpin);
        }
        while(val == 1);
      }

      if(NoteLED)
      {
        digitalWrite(LEDspin, LOW);                      // Turn on LED for column 1 (Note LED)
      }

      ColSelect = 3;
      break;

    case 3:  
      digitalWrite(LEDspin, HIGH);
      digitalWrite(COL1pin, LOW);
      digitalWrite(COL2pin, LOW);

      digitalWrite(COL3pin, HIGH);

      val = digitalRead(BUTTONpin);
      if(val == 1)
      {
        DownPress = true;
        do
        {
          val = digitalRead(BUTTONpin);
        }
        while(val == 1);
      }

      if(SelectLED)
      {
        digitalWrite(LEDspin, LOW);                      // Turn on LED for column 1 (Selected LED)
      }

      ColSelect = 1;
      break;
    }
  }
  // -----------------------------------------------------------------------------------------------------------------------

  switch (mode) 
  {
  case 1:                                                         // Normal Drum Playing 

    val = digitalRead(BUTTONpin);
    if(val == 1)
    {
      mode = 2;
      ColSelect = 2;
      ModePress = false;
      PlayLED = false;
      NoteLED = true;

      do
      {
        val = digitalRead(BUTTONpin);
      }
      while(val == 1);
    }

    for(int pin=0; pin < NumPads; pin++)
    {
      hitavg = analogRead(pin);                                   // read the input pin

        if((hitavg > PadCutOff[pin]))
      {
        if((activePad[pin] == false))
        {
          if(VelocityFlag == true)
          {
            //          hitavg = 127 / ((1023 - PadCutOff[pin]) / (hitavg - PadCutOff[pin]));    // With full range (Too sensitive ?)
            hitavg = (hitavg / 8) -1 ;                                                 // Upper range
          }
          else
          {
            hitavg = 127;
          }

          MIDI_TX(144,PadNote[pin],hitavg); 

          PinPlayTime[pin] = 0;
          activePad[pin] = true;
        }
        else
        {
          PinPlayTime[pin] = PinPlayTime[pin] + 1;
        }
      }
      else if((activePad[pin] == true))
      {
        PinPlayTime[pin] = PinPlayTime[pin] + 1;

        if(PinPlayTime[pin] > MaxPlayTime[pin])
        {
          activePad[pin] = false;
          MIDI_TX(128,PadNote[pin],127); 

        }
      }
    } 
    break;

    // ---------------------------------------------------------------------------------------

  case 2:                                                    // Re programming NOTEs Mode (wait for a Pad to be selected)

    for(int pin=0; pin < NumPads; pin++)
    {
      hitavg = analogRead(pin);                              // read the input pin

        if((hitavg > PadCutOff[pin]))
      {
        mode = 3;
        RPGpin = pin;
        SelectLED = true;
      }

      if(ModePress)                                         // User pressed ModePin again - abort NOTE change -> Play mode
      {
        mode = 1;
        ModePress = false;
        PlayLED = true;
        NoteLED = false;

        digitalWrite(LEDspin, LOW);                      // Turn on LED for column 1 (Play LED)
        digitalWrite(COL1pin, HIGH);
      }
    }

    break;

    // ---------------------------------------------------------------------------------------

  case 3:                                                         // Re programming NOTEs Mode

    if(SampON)
    {
      SampCounter = SampCounter + 1;
      if(SampCounter > SampTime)
      {
        SampON = false;
        MIDI_TX(128,PadNote[SampPlaying],0); 
      }
    }

    if(UpPress)
    {
      if(SampON)
      {
        SampON = false;
        MIDI_TX(128,PadNote[SampPlaying],0); 
      }

      PadNote[RPGpin] = PadNote[RPGpin]+1;
      if(PadNote[RPGpin] > 127)
      {
        PadNote[RPGpin] = 127;
      }
      MIDI_TX(144,PadNote[RPGpin],127); 
      //      delay(SampTime);
      //      MIDI_TX(128,PadNote[RPGpin],0); 

      //      delay(100);
      SampON = true;
      SampCounter = 0;
      UpPress = false;
    }

    if(DownPress)
    {
      if(SampON)
      {
        SampON = false;
        MIDI_TX(128,PadNote[SampPlaying],0); 
      }

      PadNote[RPGpin] = PadNote[RPGpin]-1;
      if(PadNote[RPGpin] < 1)
      {
        PadNote[RPGpin] = 1;
      }
      MIDI_TX(144,PadNote[RPGpin],127); 
      //      delay(SampTime);
      //      MIDI_TX(128,PadNote[RPGpin],0);

      //      delay(100);
      SampON = true;
      SampCounter = 0;
      DownPress = false;
    }

    hitavg = analogRead(RPGpin);                                        // read the input pin
    if((hitavg > PadCutOff[RPGpin]))
    {
      if(SampON == false)
      {
        // SampON = false;
        //  MIDI_TX(128,PadNote[SampPlaying],0); 

        MIDI_TX(144,PadNote[RPGpin],127); 
        //      delay(SampTime);
        //      MIDI_TX(128,PadNote[RPGpin],0);
        SampON = true;
        SampCounter = 0;
      }
    }

    if(ModePress)
    {
      mode = 1;

      FILLEEPROM();

      delay(100);

      ModePress = false;
      PlayLED = true;
      NoteLED = false;
      SelectLED = false;

      digitalWrite(LEDspin, LOW);                      // Turn on LED for column 1 (Play LED)
      digitalWrite(COL1pin, HIGH);
    }
    break;
  }
}


//*******************************************************************************************************************
// Transmit MIDI Message			
//*******************************************************************************************************************
void MIDI_TX(unsigned char MESSAGE, unsigned char PITCH, unsigned char VELOCITY) 
{
  status = MESSAGE + midichannel;
  Serial.write(status);
  Serial.write(PITCH);
  Serial.write(VELOCITY);
}


//*******************************************************************************************************************
// Read MIDI notes for drum pads from EEPROM			
//*******************************************************************************************************************
void GETNOTESFROMEEPROM() 
{
  int EEPadd;
  for(int EEPadd=0; EEPadd < 8; EEPadd++)
  {
    PadNote[EEPadd] = EEPROM.read(EEPadd);
  }
}


//*******************************************************************************************************************
// Save changed (or first time run) MIDI notes to EEPROM			
//*******************************************************************************************************************
void FILLEEPROM()                                          // Normally only run once if EEPROM is clear
{
  int EEPadd;
  for(int EEPadd=0; EEPadd < 8; EEPadd++)
  {
    EEPROM.write(EEPadd, PadNote[EEPadd]);
  }
  EEPROM.write(9, 78);                                      // 78 is just a number we selected to show EEPROM was writen
}



